# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2011-2013 Marc Deslauriers <marc.deslauriers@canonical.com>
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging
from pasaffe_lib.AboutDialog import AboutDialog
from pasaffe_lib import get_version

logger = logging.getLogger('pasaffe')

# pylint: disable=E1101


# See pasaffe_lib.AboutDialog.py for more details about how this class works.
class AboutPasaffeDialog(AboutDialog):
    __gtype_name__ = "AboutPasaffeDialog"

    def finish_initializing(self, builder):  # pylint: disable=E1002
        """Set up the about dialog"""
        super(AboutPasaffeDialog, self).finish_initializing(builder)

        self.set_version(get_version())
